@@ -22,7 +22,9 @@
 
 #include <stdio.h>
 #include <glib.h>
-#include <gtsconfig.h>
+#ifndef NATIVE_WIN32
+# include <gtsconfig.h>
+#endif
 #ifdef GTS_COMPILATION
 # include "config.h"
 #endif /* not GTS_COMPILATION */
@@ -171,7 +173,7 @@ void           gts_file_variable_error    (GtsFile * f,
 					   const gchar * name,
 					   const gchar * format,
 					   ...);
-void           gts_file_close             (GtsFile * f);
+void           gts_file_destroy           (GtsFile * f);
 
 /* Objects: object.c */
 
@@ -195,7 +197,8 @@ void           gts_file_close             (GtsFile * f);
 
 typedef enum
 {
-  GTS_DESTROYED         = 1 << 0
+  GTS_DESTROYED         = 1 << 0,
+  GTS_USER_FLAG         = 1 /* user flags start from here */
 } GtsObjectFlags;
 
 #define GTS_OBJECT_FLAGS(obj)             (GTS_OBJECT (obj)->flags)
@@ -301,6 +304,7 @@ void             gts_object_init                (GtsObject * object,
 						 GtsObjectClass * klass);
 void             gts_object_reset_reserved      (GtsObject * object);
 void             gts_object_destroy             (GtsObject * object);
+void             gts_finalize                   (void);
 
 /* Ranges: surface.c */
 typedef struct _GtsRange               GtsRange;
@@ -368,6 +372,10 @@ gdouble       gts_point_orientation_3d               (GtsPoint * p1,
 						      GtsPoint * p2,
 						      GtsPoint * p3,
 						      GtsPoint * p4);
+gint          gts_point_orientation_3d_sos           (GtsPoint * p1,
+						      GtsPoint * p2,
+						      GtsPoint * p3,
+						      GtsPoint * p4);
 GtsIntersect  gts_point_is_in_triangle               (GtsPoint * p,
 						      GtsTriangle * t);
 gdouble       gts_point_in_circle                    (GtsPoint * p, 
@@ -542,6 +550,8 @@ GtsEdge *     gts_edge_is_duplicate               (GtsEdge * e);
 GList *       gts_edges_merge                     (GList * edges);
 gboolean      gts_edge_belongs_to_tetrahedron     (GtsEdge * e);
 guint         gts_edge_is_contact                 (GtsEdge * e);
+void          gts_edge_swap                       (GtsEdge * e, 
+						   GtsSurface * s);
 
 /* Triangles: triangle.c */
 
@@ -898,7 +908,11 @@ struct _GtsSurfaceQualityStats {
 struct _GtsSurface {
   GtsObject object;
 
+#ifdef USE_SURFACE_BTREE
+  GTree * faces;
+#else /* not USE_SURFACE_BTREE */
   GHashTable * faces;
+#endif /* not USE_SURFACE_BTREE */
   GtsFaceClass * face_class;
   GtsEdgeClass * edge_class;
   GtsVertexClass * vertex_class;
@@ -1001,6 +1015,25 @@ void         gts_surface_distance          (GtsSurface * s1,
 					    GtsRange * boundary_range);
 GSList *     gts_surface_boundary          (GtsSurface * surface);
 
+/* Discrete differential operators: curvature.c */
+
+gboolean gts_vertex_mean_curvature_normal  (GtsVertex * v, 
+					    GtsSurface * s, 
+					    GtsVector n);
+gboolean gts_vertex_gaussian_curvature     (GtsVertex * v, 
+					    GtsSurface * s, 
+					    gdouble * Kg);
+void     gts_vertex_principal_curvatures   (gdouble Kh, 
+					    gdouble Kg, 
+					    gdouble * K1, 
+					    gdouble * K2);
+void     gts_vertex_principal_directions   (GtsVertex * v, 
+					    GtsSurface * s, 
+					    GtsVector Kh,
+                                            gdouble Kg,
+					    GtsVector e1, 
+					    GtsVector e2);
+
 /* Volume optimization: vopt.c */
 typedef struct _GtsVolumeOptimizedParams   GtsVolumeOptimizedParams;
 
@@ -1053,9 +1086,12 @@ gts_surface_inter_new            (GtsSurfaceInterClass * klass,
 				  GtsSurface * s1,
 				  GtsSurface * s2,
 				  GNode * faces_tree1,
-				  GNode * faces_tree2);
+				  GNode * faces_tree2,
+				  gboolean is_open1,
+				  gboolean is_open2);
 gboolean 
-gts_surface_inter_check          (GtsSurfaceInter * si);
+gts_surface_inter_check          (GtsSurfaceInter * si,
+				  gboolean * closed);
 void 
 gts_surface_inter_boolean        (GtsSurfaceInter * si, 
 				  GtsSurface * surface,
@@ -1394,7 +1430,7 @@ guint                gts_delaunay_refine         (GtsSurface * surface,
 						  GtsKeyFunc cost,
 						  gpointer cost_data);
 
-/* Isosurfaces: iso.c */
+/* Isosurfaces (marching cubes): iso.c */
 
 typedef struct _GtsGridPlane     GtsGridPlane;
 typedef struct _GtsIsoSlice      GtsIsoSlice;
@@ -1442,6 +1478,20 @@ void           gts_isosurface_cartesian     (GtsSurface * surface,
 					     gpointer data,
 					     gdouble iso);
 
+/* Isosurfaces (marching tetrahedra): isotetra.c */
+
+void           gts_isosurface_tetra         (GtsSurface * surface,
+					     GtsCartesianGrid g,
+					     GtsIsoCartesianFunc f,
+					     gpointer data,
+					     gdouble iso);
+
+void           gts_isosurface_tetra_bounded (GtsSurface * surface,
+					     GtsCartesianGrid g,
+					     GtsIsoCartesianFunc f,
+					     gpointer data,
+					     gdouble iso);
+
 /* Named vertices, edges and triangles: named.c */
 
 #define GTS_NAME_LENGTH             40
@@ -1619,7 +1669,9 @@ struct _GtsContaineeClass {
 
   void     (* add_container)    (GtsContainee *, GtsContainer *);
   void     (* remove_container) (GtsContainee *, GtsContainer *);
+  void     (* foreach)          (GtsContainee *, GtsFunc, gpointer);
   gboolean (* is_contained)     (GtsContainee *, GtsContainer *);
+  void     (* replace)          (GtsContainee *, GtsContainee *);
 };
 
 #define GTS_CONTAINEE(obj)            GTS_OBJECT_CAST (obj,\
@@ -1635,6 +1687,8 @@ GtsContaineeClass * gts_containee_class        (void);
 GtsContainee *      gts_containee_new          (GtsContaineeClass * klass);
 gboolean            gts_containee_is_contained (GtsContainee * item, 
 						GtsContainer * c);
+void                gts_containee_replace      (GtsContainee * item,
+						GtsContainee * with);
 
 /* GtsSListContainee: container.c */
 
@@ -1768,6 +1822,7 @@ struct _GtsGNodeClass {
   GtsSListContainerClass parent_class;
 
   gfloat (* weight) (GtsGNode *);
+  void   (* write)  (GtsGNode *, FILE *);
 };
 
 #define GTS_GNODE(obj)            GTS_OBJECT_CAST (obj,\
@@ -1856,6 +1911,34 @@ GtsWGNodeClass * gts_wgnode_class                (void);
 GtsWGNode *      gts_wgnode_new                  (GtsWGNodeClass * klass,
 						  gfloat weight);
 
+/* GtsPNode */
+
+typedef struct _GtsPNode         GtsPNode;
+typedef struct _GtsPNodeClass    GtsPNodeClass;
+
+struct _GtsPNode {
+  GtsGNode node;
+
+  gpointer data;
+};
+
+struct _GtsPNodeClass {
+  GtsGNodeClass parent_class;
+};
+
+#define GTS_PNODE(obj)            GTS_OBJECT_CAST (obj,\
+					           GtsPNode,\
+					           gts_pnode_class ())
+#define GTS_PNODE_CLASS(klass)    GTS_OBJECT_CLASS_CAST (klass,\
+						         GtsPNodeClass,\
+						         gts_pnode_class())
+#define GTS_IS_PNODE(obj)         (gts_object_is_from_class (obj,\
+						   gts_pnode_class ()))
+     
+GtsPNodeClass * gts_pnode_class                (void);
+GtsPNode *      gts_pnode_new                  (GtsPNodeClass * klass,
+						gpointer data);
+
 /* GtsFNode */
 
 typedef struct _GtsFNode         GtsFNode;
@@ -1899,8 +1982,9 @@ struct _GtsGEdge {
 struct _GtsGEdgeClass {
   GtsContaineeClass parent_class;
 
-  GtsGEdge * (* link) (GtsGEdge * e, GtsGNode * n1, GtsGNode * n2);
-  gfloat (* weight)   (GtsGEdge * e);
+  GtsGEdge * (* link)   (GtsGEdge * e, GtsGNode * n1, GtsGNode * n2);
+  gfloat     (* weight) (GtsGEdge * e);
+  void       (* write)  (GtsGEdge * e, FILE * fp);
 };
 
 #define GTS_GEDGE(obj)            GTS_OBJECT_CAST (obj,\
@@ -1920,6 +2004,36 @@ gfloat          gts_gedge_weight               (GtsGEdge * e);
 #define         gts_gedge_connects(e, a1, a2)\
    (((e)->n1 == a1 && (e)->n2 == a2) || ((e)->n1 == a2 && (e)->n2 == a1)) 
 
+/* GtsPGEdge: graph.c */
+
+typedef struct _GtsPGEdge         GtsPGEdge;
+typedef struct _GtsPGEdgeClass    GtsPGEdgeClass;
+
+struct _GtsPGEdge {
+  GtsGEdge gedge;
+
+  gpointer data;
+};
+
+struct _GtsPGEdgeClass {
+  GtsGEdgeClass parent_class;
+};
+
+#define GTS_PGEDGE(obj)            GTS_OBJECT_CAST (obj,\
+					           GtsPGEdge,\
+					           gts_pgedge_class ())
+#define GTS_PGEDGE_CLASS(klass)    GTS_OBJECT_CLASS_CAST (klass,\
+						         GtsPGEdgeClass,\
+						         gts_pgedge_class())
+#define GTS_IS_PGEDGE(obj)         (gts_object_is_from_class (obj,\
+						   gts_pgedge_class ()))
+     
+GtsPGEdgeClass * gts_pgedge_class                (void);
+GtsPGEdge *      gts_pgedge_new                  (GtsPGEdgeClass * klass,
+						  GtsGNode * n1,
+						  GtsGNode * n2,
+						  gpointer data);
+
 /* GtsWGEdge: graph.c */
 
 typedef struct _GtsWGEdge         GtsWGEdge;
@@ -2002,6 +2116,8 @@ guint              gts_graph_edges_cut           (GtsGraph * g);
 gfloat             gts_graph_edges_cut_weight    (GtsGraph * g);
 void               gts_graph_write               (GtsGraph * g, 
 						  FILE * fp);
+void               gts_graph_write_dot           (GtsGraph * g, 
+						  FILE * fp);
 guint              gts_graph_read                (GtsGraph * g, 
 						  GtsFile * fp);
 guint              gts_graph_read_jostle         (GtsGraph * g, 
@@ -2041,6 +2157,11 @@ GtsGraph *       gts_surface_graph_new           (GtsGraphClass * klass,
 GtsSurface *     gts_surface_graph_surface       (GtsGraph * surface_graph,
 						  GtsSurface * s);
 
+/* Segments graph: graph.c */
+
+GtsGraph *       gts_segments_graph_new          (GtsGraphClass * klass,
+						  GSList * segments);
+
 /* GtsGNodeSplit: pgraph.c */
 
 typedef struct _GtsGNodeSplit         GtsGNodeSplit;
